/*
  Copyright (C) 2010 DJ Delorie <dj@redhat.com>

  This file is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2, or (at your option) any later
  version.
  
  This file is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with this file; see the file COPYING3.  If not see
  <http://www.gnu.org/licenses/>.
*/

#include "sfr_r8c33c.h"
#include "leds.h"

extern unsigned int led_gamma[];

extern void timer_rb_hook();
void *j = &timer_rb_hook;

volatile struct {
  int val;
  int cur;
} led_vals[4];

static led_save[4];

static void __attribute__((constructor))
init_leds ()
{
  trcmr  = 0x0f;
  trccr1 = 0x00;
  trccr2 = 0x00;

  trc = 0;
  trcgrb = 0;
  trcgrc = 0;
  trcgrd = 0;

  trcoer = 0x00;
  trcpsr0 = 0x50;	/* TRCIOB on P2.0 */
  trcpsr1 = 0x44;	/* C on P2.1, D on P2.2 */

  tstart_trcmr = 1;

  
  trbcr = 0;
  trbmr = 0;		/* f1 = 18432 KHz */
  trbpre = trbsc = 29;
  trbpr = 39;
  trbioc = 0;

  trbic = 5;
  tstart_trbcr = 1;
}

static int rv, gv, bv;

void
set_rgb (int r, int g, int b)
{
  rv = r;
  gv = g;
  bv = b;
  tstart_trcmr = 0;
  trcgrd = led_gamma[r & 0xff];
  trcgrc = led_gamma[g & 0xff];
  trcgrb = led_gamma[b & 0xff];
  tstart_trcmr = 1;
}

void
get_rgb (int *r, int *g, int *b)
{
  *r = rv;
  *g = gv;
  *b = bv;
}

void
set_leds (int bitmask)
{
  set_led (0, (bitmask & 1) ? 255 : 0);
  set_led (1, (bitmask & 2) ? 255 : 0);
  set_led (2, (bitmask & 4) ? 255 : 0);
  set_led (3, (bitmask & 8) ? 255 : 0);
}

void
set_led (int which, int val)
{
  if (0 <= which && which <= 3)
    {
      led_save[which] = val;
      led_vals[which].val = led_gamma[val & 0xff];
    }
}

int
get_led (int which)
{
  return led_save[which];
}
