/*
  Copyright (C) 2010 DJ Delorie <dj@redhat.com>

  This file is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2, or (at your option) any later
  version.
  
  This file is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with this file; see the file COPYING3.  If not see
  <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include "sfr_r8c33c.h"
#include "tty.h"
#include "leds.h"

#define printf cprintf

extern unsigned int rand;

const char * const colorstrings[] = {
  "\033[31mRED ",
  "\033[32mGRN ",
  "\033[33mYEL ",
  "\033[34mBLU ",
  "\033[35mMAG ",
  "\033[36mCYN "
};

const char backspace_string[] =
  "\b\b\b\b    \b\b\b\b";

#define BEEP '\a'

char colors[4];
char guess[4];
char exact, inexact;

char did_e=0, did_i=0, did_ie=0, did_none=0;

static int
getkey ()
{
  int i;
  unsigned char p;
  while (!tty_ready ())
    rand ++;
  return tty_getc ();
}

print_banner ()
{
  printf("\n");
  printf("Welcome to Micro Mind!  Guess the Four Colors!\n");
  printf("\n");
  printf("Press \033[31mR \033[32mG \033[34mB \033[36mC \033[35mM \033[33mY \033[0mfor colors, backspace, or enter\n");
  printf("After each guess, there's one \033[32m#\033[0m for each correct color\n");
  printf("in the correct location, and one \033[33m?\033[0m for each correct color\n");
  printf("in the wrong location.  Press ESC to give up.  Note: colors may be used more\n");
  printf("than once in any given solution.\n");
  printf("\n");
}

choose_colors ()
{
  int i, r;
  r = rand;
  for (i=0; i<4; i++)
    {
      colors[i] = rand % 6;
      rand /= 6;
    }
}

int
obtain_guess ()
{
  char buf[11];
  int i, j=0, g;
  printf("Guess ? ");

  while (1)
    switch (getkey ())
      {
      case 'r':
      case 'R':
	g = 0;
        goto do_guess;
      case 'g':
      case 'G':
	g = 1;
        goto do_guess;
      case 'y':
      case 'Y':
	g = 2;
        goto do_guess;
      case 'b':
      case 'B':
	g = 3;
        goto do_guess;
      case 'm':
      case 'M':
	g = 4;
        goto do_guess;
      case 'c':
      case 'C':
	g = 5;
        goto do_guess;

      case 'w':
	set_rgb (255, 255, 255);
	break;

      do_guess:
	if (j > 3)
	  tty_putc(BEEP);
	else
	  {
	    set_rgb ((g+1)&1 ? 255 : 0,
		     (g+1)&2 ? 255 : 0,
		     (g+1)&4 ? 255 : 0);
	    tty_puts (colorstrings[g]);
	    guess[j++] = g;
	  }
	break;

      case '\b':
      case 127:
	set_rgb (0, 0, 0);
	if (j == 0)
	  tty_putc (BEEP);
	else
	  {
	    j --;
	    tty_puts (backspace_string);
	  }
        break;

      case 27:
	set_rgb (0, 0, 0);
	printf("\nGiving up?  I was thinking of ");
	for (j=0; j<4; j++)
	  tty_puts (colorstrings[colors[j]]);
	tty_puts("\033[0m\n\n");
	return 1;

      case '\r':
      case '\n':
	set_rgb (0, 0, 0);
	if (j == 4)
	  {
	    tty_puts ("\033[0m");
	    return 0;
	  }
        tty_putc (BEEP);
        break;
      }
}

static const char *
s(int i)
{
  return i == 1 ? "" : "s";
}

int
print_results ()
{
  char gs[4], ss[4];
  int x, y, l = 0;

  for (x=0; x<4; x++)
    {
      gs[x] = ss[x] = 0;
      set_led (x, 0);
    }

  exact = inexact = 0;

  for (x=0; x<4; x++)
    if (guess[x] == colors[x])
      {
	exact ++;
	set_led (l++, 255);
	gs[x] = ss[x] = 1;
      }
  for (x=0; x<4; x++)
    if (gs[x] == 0)
      for (y=0; y<4; y++)
	if (ss[y] == 0)
	  if (guess[x] == colors[y])
	    {
	      inexact ++;
	      set_led (l++, 64);
	      gs[x] = ss[y] = 1;
	      break;
	    }

  printf("  \033[32m");
  for (x=0; x<exact; x++)
    printf(" #");
  printf("\033[33m");
  for (x=0; x<inexact; x++)
    printf(" ?");
  if (exact == 0 && inexact == 0)
    printf(" \033[31mX");

  tty_puts ("\033[0m");
  switch ((exact ? 2 : 0) | (inexact ? 1 : 0))
    {
    case 0:
      if (!did_none)
	{
	  printf("\n  (that means none of those colors are used)");
	  did_none = 1;
	}
      break;
    case 1:
      if (!did_i)
	{
	  printf("\n  (that means %d color%s %s right, but in the wrong place)",
		 inexact, s(inexact), inexact == 1 ? "is" : "are");
	  did_i = 1;
	}
      break;
    case 2:
      if (!did_e)
	{
	  printf("\n  (that means %d color%s %s right, and in the right place)",
		 exact, s(exact), exact == 1 ? "is" : "are");
	  did_e = 1;
	}
      break;
    case 3:
      if (!did_ie)
	{
	  printf("\n  (that means %d color%s %s in the right place, and %d other%s %s in the wrong place)",
		 exact, s(exact), exact == 1 ? "is" : "are",
		 inexact, s(inexact), inexact == 1 ? "is" : "are");
	  did_ie = 1;
	}
      break;
    }

  tty_putc('\n');

  return (exact < 4);
}

void
micromind ()
{
  print_banner ();

  choose_colors ();

  do
    {
      if (obtain_guess ())
	goto start_again;
    }
  while (print_results ());
  
  printf("\nCorrect!  ");
  set_rgb (255, 255, 255);
 start_again:
  printf("Let's play again soon!\n");
}
