/*
  Copyright (C) 2010 DJ Delorie <dj@redhat.com>

  This file is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation; either version 2, or (at your option) any later
  version.
  
  This file is distributed in the hope that it will be useful, but WITHOUT ANY
  WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with this file; see the file COPYING3.  If not see
  <http://www.gnu.org/licenses/>.
*/

#include "sfr_r8c33c.h"
#include "tty.h"

unsigned int rand;

static char buf[16];
static unsigned char bufp, bufg;
static volatile int bufc;

/* nRTS is the only input */
#define nRTS p1_6

#define nCTS p1_7

#define STOP()	nCTS = 1
#define START()	nCTS = 0

static int do_flow_control = 0;

void __attribute__((interrupt))
s0rx_handler ()
{
  buf[bufp&15] = u0rb;
  bufp ++;
  bufc ++;
  if (bufc > 12)
    STOP();
}

void
tty_init ()
{
  pd1_4 = 1; /* TXD */
  pd1_5 = 0; /* RXD */
  pd1_6 = 0; /* RTS */
  pd1_7 = 1; /* CTS */

  START();

  u0brg = 18432000 / (115200 * 16) - 1;

  u0mr = 5;	/* UART, 8N1 */
  u0c0 = 0;	/* LSB first, CMOS, buffered, f1 */
  u0c1 = 5;	/* enable tx, rx */
  u0sr = 5;

  s0ric = 5;

  wait_ms (1);

  if (nRTS)
    do_flow_control = 0;
  else
    do_flow_control = 1;
}

void
tty_flush ()
{
  while (bufc)
    rand ++;
}

void
tty_putc (char c)
{
  int i;
  if (c == '\n')
    tty_putc ('\r');
  while (do_flow_control && nRTS)
    ;
  while (!ti_u0c1)
    rand ++;
  u0tb = c;
}

void
tty_puts (const char *s)
{
  while (*s)
    {
      tty_putc (*s);
      s++;
    }
}

void
tty_write (void *data, int len)
{
  char *d = (char *)data;
  while (len--)
    tty_putc (*d++);
}

write (int fd, void *data, int len)
{
  tty_write (data, len);
  return len;
}

/* returns number of characters available */
int
tty_ready (void)
{
  return bufc;
}

/* returns character */
int
tty_getc (void)
{
  int rv;
#if 1
  while (bufc == 0)
    rand ++;
  rv = buf[bufg&15];
  bufg++;
  asm("fclr I");
  bufc--;
  if (bufc < 4)
    START();
  asm("fset I");
#else
  while (u0c1.ri)
    rand ++;
  rv = u0rb;
#endif
  return rv;
}

#if 0
/* returns number of characters read */
int
tty_read (void *data, int len)
{
}
#endif
